@@ -18,11 +18,10 @@
 #' 
 #' \dontrun{
 #' pub <- get_publication("refs-201819-03-06")
-#' (pub %>% html_elements("replikk"))[1] %>% html_text()
+#' (pub |> html_elements("replikk"))[1] |> html_text()
 #' }
-#' 
 #'  
-#' @import rvest httr
+#' @import rvest httr2
 #' 
 #' @export
 #' 
@@ -30,15 +29,40 @@ get_publication <- function(publicationid = NA, good_manners = 0){
   
   url <- paste0("https://data.stortinget.no/eksport/publikasjon?publikasjonid=", publicationid)
   
-  base <- GET(url)
+  base <- request(url)
   
-  resp <- http_type(base)
-  if(resp != "text/xml") stop(paste0("Response of ", url, " is not text/xml."), call. = FALSE)
+  resp <- base |> 
+    req_error(is_error = function(resp) FALSE) |> 
+    req_perform()
   
-  status <- http_status(base)
-  if(status$category != "Success") stop(paste0("Response of ", url, " returned as '", status$message, "'"), call. = FALSE)
+  if(resp$status_code != 200) {
+    stop(
+      paste0(
+        "Response of ", 
+        url, 
+        " is '", 
+        resp |> resp_status_desc(),
+        "' (",
+        resp$status_code,
+        ")."
+      ), 
+      call. = FALSE)
+  }
   
-  tmp <- read_html(base)
+  if(resp_content_type(resp) != "text/xml") {
+    stop(
+      paste0(
+        "Response of ", 
+        url, 
+        " returned as '", 
+        resp_content_type(resp), 
+        "'.",
+        " Should be 'text/xml'."), 
+      call. = FALSE) 
+  }
+  
+  tmp <- resp |> 
+    resp_body_html(check_type = FALSE, encoding = "utf-8") 
 
   return(tmp)
   
